@@ -16,6 +16,7 @@
 import hashlib
 import json
 import os
+import random
 import shutil
 import signal
 import subprocess
@@ -23,6 +24,7 @@ import sys
 import time
 
 from oslo_config import cfg
+from oslo_log import log
 
 from os_collect_config import cache
 from os_collect_config import cfn
@@ -32,7 +34,6 @@ from os_collect_config import heat
 from os_collect_config import heat_local
 from os_collect_config import keystone
 from os_collect_config import local
-from os_collect_config.openstack.common import log
 from os_collect_config import request
 from os_collect_config import version
 from os_collect_config import zaqar
@@ -64,9 +65,14 @@ opts = [
                 help='Pass this option to make os-collect-config exit after'
                 ' one execution of command. This behavior is implied if no'
                 ' command is specified.'),
+    cfg.FloatOpt('min-polling-interval', default=1,
+                 help='When running continuously, pause a minimum of this'
+                      ' many seconds between collecting data.'),
     cfg.FloatOpt('polling-interval', short='i', default=30,
-                 help='When running continuously, pause this many seconds'
-                      ' between collecting data.'),
+                 help='When running continuously, pause a maximum of this'
+                      ' many seconds between collecting data. If changes'
+                      ' are detected shorter sleeps intervals are gradually'
+                      ' increased to this maximum polling interval.'),
     cfg.BoolOpt('print-cachedir',
                 default=False,
                 help='Print out the value of cachedir and exit immediately.'),
@@ -83,8 +89,18 @@ opts = [
                     default=['deployments'],
                     help='Key(s) to explode into multiple collected outputs. '
                     'Parsed according to the expected Metadata created by '
-                    'OS::Heat::StructuredDeployment. Only Exploded if seen at '
-                    'the root of the Metadata.')
+                    'OS::Heat::StructuredDeployment. Only exploded if seen at '
+                    'the root of the Metadata.'),
+    cfg.FloatOpt('splay',
+                 default=0,
+                 help='Use this option to sleep for a random amount of time '
+                      'prior to starting the collect process. Takes a maximum '
+                      'number of seconds to wait before beginning collection '
+                      'as an argument. Disabled when set to 0. This option '
+                      'can help ensure that multiple collect processes '
+                      '(on different hosts) do not attempt to poll at the '
+                      'exact same time if they were all started at the same '
+                      'time. Ignored if --one-time or --force is used.'),
 ]
 
 CONF = cfg.CONF
@@ -142,6 +158,7 @@ def setup_conf():
     CONF.register_cli_opts(zaqar.opts, group='zaqar')
 
     CONF.register_cli_opts(opts)
+    log.register_options(CONF)
 
 
 def collect_all(collectors, store=False, collector_kwargs_map=None):
@@ -232,7 +249,7 @@ def __main__(args=sys.argv, collector_kwargs_map=None):
     # output in tests cleanly, so should only be called if there isn't already
     # handlers defined i.e. not in unit tests
     if not log.getLogger(None).logger.handlers:
-        log.setup("os-collect-config")
+        log.setup(CONF, "os-collect-config")
 
     if CONF.print_cachedir:
         print(CONF.cachedir)
@@ -247,10 +264,18 @@ def __main__(args=sys.argv, collector_kwargs_map=None):
     if CONF.force:
         CONF.set_override('one_time', True)
 
+    if CONF.splay > 0 and not CONF.one_time:
+        # sleep splay seconds in the beginning to prevent multiple collect
+        # processes from all running at the same time
+        time.sleep(random.randrange(0, CONF.splay))
+
     exitval = 0
     config_files = CONF.config_file
     config_hash = getfilehash(config_files)
+    exponential_sleep_time = CONF.min_polling_interval
     while True:
+        # shorter sleeps while changes are detected allows for faster
+        # software deployment dependency processing
         store_and_run = bool(CONF.command and not CONF.print_only)
         (changed_keys, content) = collect_all(
             cfg.CONF.collectors,
@@ -266,32 +291,24 @@ def __main__(args=sys.argv, collector_kwargs_map=None):
                     exitval = e.returncode
                     logger.error('Command failed, will not cache new data. %s'
                                  % e)
-                    if not CONF.one_time:
-                        new_config_hash = getfilehash(config_files)
-                        if config_hash == new_config_hash:
-                            logger.warn(
-                                'Sleeping %.2f seconds before re-exec.' %
-                                CONF.polling_interval
-                            )
-                            time.sleep(CONF.polling_interval)
-                        else:
-                            # The command failed but the config file has
-                            # changed re-exec now as the config file change
-                            # may have fixed things.
-                            logger.warn('Config changed, re-execing now')
-                            config_hash = new_config_hash
                 else:
                     for changed in changed_keys:
                         cache.commit(changed)
                 if not CONF.one_time:
-                    reexec_self()
+                    new_config_hash = getfilehash(config_files)
+                    if config_hash != new_config_hash:
+                        reexec_self()
             else:
                 logger.debug("No changes detected.")
             if CONF.one_time:
                 break
             else:
-                logger.info("Sleeping %.2f seconds.", CONF.polling_interval)
-                time.sleep(CONF.polling_interval)
+                logger.info("Sleeping %.2f seconds.", exponential_sleep_time)
+                time.sleep(exponential_sleep_time)
+
+            exponential_sleep_time *= 2
+            if exponential_sleep_time > CONF.polling_interval:
+                exponential_sleep_time = CONF.polling_interval
         else:
             print(json.dumps(content, indent=1))
             break
